translation_tc = {
    "Add Bible Tab": "新聖經標籤頁",
    "Add Parallel Tab": "新平行標籤頁",
    "Add Tool Tab": "新工具標籤頁",
    "Agent Mode": "代理模式",
    "Agent": "代理",
    "AI Chat": "對話模式",
    "AI Commentary": "AI 註釋",
    "Ask BibleMate": "問 BibleMate",
    "Bible Audio": "聖經語音",
    "Bible Chronology": "聖經時序",
    "Bible Commentaries": "聖經註釋",
    "Bible Only": "只顯示聖經",
    "Bible Parallels": "平行經文",
    "Bible Podcast": "聖經廣播",
    "Bible Promises": "聖經應許",
    "Bible": "聖經",
    "Bibles": "聖經",
    "Parallel Bibles": "平行聖經",
    "Timelines": "聖經時間軸",
    "Bible Timelines": "聖經時間軸",
    "Characters": "聖經人物",
    "Close Bible Tab": "關閉聖經標籤頁",
    "Close Others": "關閉其他",
    "Close Parallel Tab": "關閉平行標籤頁",
    "Close Tool Tab": "關閉工具標籤頁",
    "Commentaries": "聖經註釋",
    "Comparison": "版本對比",
    "Copy": "複製",
    "Cross-references": "經文串珠",
    "Dictionaries": "聖經字典",
    "Discourse Analysis": "語法分析",
    "Encyclopedias": "百科全書",
    "Go": "開啓",
    "Indexes": "資源索引",
    "Chapter Indexes": "全章索引",
    "Verse Indexes": "單節索引",
    "Lexicons": "原文詞典",
    "Locations": "聖經地點",
    "Maps": "聖經地圖",
    "Morphological Data": "形態學數據",
    "Morphology": "形態學分析",
    "Names": "聖經名字",
    "Parallels": "平行經文",
    "Partner Mode": "搭檔模式",
    "Preferences": "偏好設定",
    "Promises": "聖經應許",
    "Relationships": "人物關係",
    "Screen": "全螢幕",
    "Fullscreen": "全螢幕",
    "Swap": "版面轉換",
    "Sync": "同步",
    "Tool Only": "只顯示工具",
    "Topics": "聖經主題",
    "Treasury": "經文寶庫",
    "Treasury of Scripture Knowledge": "經文寶庫",
    "Bible & Tool": "聖經與工具",
    "Light Mode": "淺色模式",
    "Dark Mode": "深色模式",
    "Verses": "搜索經文",
    "Tools": "工具",
    "Search": "搜索",
    "Quick search": "快速搜索",
    "AI": "人工智能",
    "Settings": "設定",
    "Swap Layout": "版面轉換",
    "Select Bible Verse": "聖經選單",
    "Toggle Display of Swap Layout Button": "[開啓/關閉] 版面轉換按鈕",
    "Toggle Display of Bible Selection Button": "[開啓/關閉] 聖經選單按鈕",
    "Toggle Bible Synchronization": "[開啓/關閉] 聖經同步功能",
    "Toggle Fullscreen": "[開啓/關閉] 全螢幕",
    "Prev Chapter": "上一章",
    "Next Chapter": "下一章",
    "OT": "舊約",
    "NT": "新約",
    "Select Book": "選擇書卷",
    "Select Chapter": "選擇章數",
    "Select Verse": "選擇節數",
    "Back": "返回",
    "Share": "分享",
    "Note": "筆記",
    "Notes": "筆記",
    "Bible Notes": "聖經筆記",
    "Notepad": "筆記本",
    "Add Note": "新增筆記",
    "Edit Note": "編輯筆記",
    "Save": "儲存",
    "Download": "下載",
    "Import": "匯入",
    "Delete": "刪除",
    "Logout": "登出",
    "Read": "閱讀",
    "Edit": "編輯",
    "Clear": "清除",
    "Download Index": "下載索引",
    "Rebuild Index": "重建索引",
    "Download All": "下載所有筆記",
    "Book Note": "書卷筆記",
    "Chapter Note": "全章筆記",
    "Verse Note": "經文筆記",
    "Book Tools": "書卷工具",
    "Chapter Tools": "全章工具",
    "Verse Tools": "單節工具",
    "Chapter Summary": "全章摘要",
    "Verse Commentary": "經文註釋",
    "Book Analysis": "書卷分析",
    "Overview": "概述",
    "Structural Outline": "結構大綱",
    "Logical Flow": "書卷脈絡",
    "Historical Setting": "歷史背景",
    "Themes": "主題",
    "Keywords": "關鍵詞",
    "Theology": "神學",
    "Canonical Placement": "正典位置",
    "Practical Living": "生活應用",
    "Summary": "摘要",
    "Chapter Map": "全章地圖",
    "Loop": "循環",
    "Edit Mode": "編輯模式",
    "Read Mode": "閱讀模式",
    "Literal": "字面",
    "Regex": "模式",
    "Semantic": "語義",
    "Case-sensitive": "大小寫",
    "Type to filter": "輸入以篩選",
    "results": "結果",
    "result": "結果",
    "Search for words or refs": "搜索經文或參照（如：申 6:4; 約 3:16-18）",
    "Search Bible dictionaries": "搜索聖經字典",
    "Search for a bible topic": "搜索聖經主題",
    "Search for bible promises": "搜索聖經應許",
    "Search for parallel passages": "搜索平行經文",
    "Search for a name or meaning": "搜索名字或含義",
    "Search for location(s) or verse reference(s)": "搜索地點或經文參照",
    "Search for a bible location": "搜索聖經地點",
    "Search for a bible character": "搜索聖經人物",
    "Show Verses": "顯示所有經文",
    "Distance": "距離",
    "From": "從",
    "To": "到",
    "km": "公里",
    "miles": "英里",
    "Loading...": "載入中...",
    "Connecting...": "連線中...",
    "Canceling...": "取消中...",
    "Running...": "執行中...",
    "Stopping...": "停止中...",
    "Stop running task?": "停止執行中的工作？",
    "Enter bible verse reference(s) here...": "在此輸入經文參照...",
    "Start typing your notes here...": "在此開始輸入您的筆記...",
    "Reset All Preferences": "重置所有偏好設定",
    "Language": "語言",
    "Delete": "刪除",
    "Cancel": "取消",
    "Cancelled!": "已取消！",
    "Done!": "完成！",
    "Stop": "停止",
    "Start": "開始",
    "Send": "發送",
    "Auto-scroll": "自動滾動",
    "Improve Prompt": "優化指示",
    "Semantic Searches": "語義搜索",
    "Frequently Used Resources": "常用資源",
    "User & Custom Data": "用戶與自定數據",
    "Tabs Management": "標籤頁管理",
    "Appearance": "外觀",
    "Enter your message...": "輸入您的訊息...",
    "Creating agent...": "創建代理中...",
    "Created!": "已創建！",
    "Error": "錯誤",
}