translation_sc = {
    "Add Bible Tab": "新圣经标签页",
    "Add Parallel Tab": "新平行标签页",
    "Add Tool Tab": "新工具标签页",
    "Agent Mode": "代理模式",
    "Agent": "代理",
    "AI Chat": "对话模式",
    "AI Commentary": "AI 注释",
    "Ask BibleMate": "问 BibleMate",
    "Bible Audio": "圣经语音",
    "Bible Chronology": "圣经时序",
    "Bible Commentaries": "圣经注释",
    "Bible Only": "只显示圣经",
    "Bible Parallels": "平行经文",
    "Bible Podcast": "圣经广播",
    "Bible Promises": "圣经应许",
    "Bible": "圣经",
    "Bibles": "圣经",
    "Parallel Bibles": "平行圣经",
    "Timelines": "圣经时间轴",
    "Bible Timelines": "圣经时间轴",
    "Characters": "圣经人物",
    "Close Bible Tab": "关闭圣经标签页",
    "Close Others": "关闭其他",
    "Close Parallel Tab": "关闭平行标签页",
    "Close Tool Tab": "关闭工具标签页",
    "Commentaries": "圣经注释",
    "Comparison": "版本对比",
    "Copy": "复制",
    "Cross-references": "经文串珠",
    "Dictionaries": "圣经字典",
    "Discourse Analysis": "语法分析",
    "Encyclopedias": "百科全书",
    "Go": "开启",
    "Indexes": "资源索引",
    "Chapter Indexes": "全章索引",
    "Verse Indexes": "单节索引",
    "Lexicons": "原文词典",
    "Locations": "圣经地点",
    "Maps": "圣经地图",
    "Morphological Data": "形态学数据",
    "Morphology": "形态学分析",
    "Names": "圣经名字",
    "Parallels": "平行经文",
    "Partner Mode": "搭档模式",
    "Preferences": "偏好设定",
    "Promises": "圣经应许",
    "Relationships": "人物关系",
    "Screen": "全萤幕",
    "Fullscreen": "全萤幕",
    "Swap": "版面转换",
    "Sync": "同步",
    "Tool Only": "只显示工具",
    "Topics": "圣经主题",
    "Treasury": "经文宝库",
    "Treasury of Scripture Knowledge": "经文宝库",
    "Bible & Tool": "圣经与工具",
    "Light Mode": "浅色模式",
    "Dark Mode": "深色模式",
    "Verses": "搜索经文",
    "Tools": "工具",
    "Search": "搜索",
    "Quick search": "快速搜索",
    "AI": "人工智能",
    "Settings": "设定",
    "Swap Layout": "版面转换",
    "Select Bible Verse": "圣经选单",
    "Toggle Display of Swap Layout Button": "[开启/关闭] 版面转换按钮",
    "Toggle Display of Bible Selection Button": "[开启/关闭] 圣经选单按钮",
    "Toggle Bible Synchronization": "[开启/关闭] 圣经同步功能",
    "Toggle Fullscreen": "[开启/关闭] 全萤幕",
    "Prev Chapter": "上一章",
    "Next Chapter": "下一章",
    "OT": "旧约",
    "NT": "新约",
    "Select Book": "选择书卷",
    "Select Chapter": "选择章数",
    "Select Verse": "选择节数",
    "Back": "返回",
    "Share": "分享",
    "Note": "笔记",
    "Notes": "笔记",
    "Bible Notes": "圣经笔记",
    "Notepad": "笔记本",
    "Add Note": "新增笔记",
    "Edit Note": "编辑笔记",
    "Save": "保存",
    "Download": "下载",
    "Import": "导入",
    "Delete": "删除",
    "Logout": "登出",
    "Read": "阅读",
    "Edit": "编辑",
    "Clear": "清除",
    "Download Index": "下载索引",
    "Rebuild Index": "重建索引",
    "Download All": "下载所有笔记",
    "Book Note": "书卷笔记",
    "Chapter Note": "全章笔记",
    "Verse Note": "经文笔记",
    "Book Tools": "书卷工具",
    "Chapter Tools": "全章工具",
    "Verse Tools": "单节工具",
    "Chapter Summary": "全章摘要",
    "Verse Commentary": "经文注释",
    "Book Analysis": "书卷分析",
    "Overview": "概述",
    "Structural Outline": "结构大纲",
    "Logical Flow": "书卷脉络",
    "Historical Setting": "历史背景",
    "Themes": "主题",
    "Keywords": "关键字",
    "Theology": "神学",
    "Canonical Placement": "正典位置",
    "Practical Living": "生活应用",
    "Summary": "摘要",
    "Chapter Map": "全章地图",
    "Loop": "循环",
    "Edit Mode": "编辑模式",
    "Read Mode": "阅读模式",
    "Literal": "字面",
    "Regex": "模式",
    "Semantic": "语义",
    "Case-sensitive": "大小写",
    "Type to filter": "输入以筛选",
    "results": "结果",
    "result": "结果",
    "Search for words or refs": "搜索经文或参照（如：申 6:4; 约 3:16-18）",
    "Search Bible dictionaries": "搜索圣经字典",
    "Search for a bible topic": "搜索圣经主题",
    "Search for bible promises": "搜索圣经应许",
    "Search for parallel passages": "搜索平行经文",
    "Search for a name or meaning": "搜索名字或含义",
    "Search for location(s) or verse reference(s)": "搜索地点或经文参照",
    "Search for a bible location": "搜索圣经地点",
    "Search for a bible character": "搜索圣经人物",
    "Show Verses": "显示所有经文",
    "Distance": "距离",
    "From": "从",
    "To": "到",
    "km": "公里",
    "miles": "英里",
    "Loading...": "载入中...",
    "Connecting...": "连接中...",
    "Canceling...": "取消中...",
    "Running...": "运行中...",
    "Stopping...": "停止中...",
    "Stop running task?": "停止运行中的工作？",
    "Enter bible verse reference(s) here...": "在此输入经文参照...",
    "Start typing your notes here...": "在此开始输入您的笔记...",
    "Reset All Preferences": "重置所有偏好设定",
    "Language": "语言",
    "Delete": "删除",
    "Cancel": "取消",
    "Cancelled!": "已取消！",
    "Done!": "完成！",
    "Stop": "停止",
    "Start": "开始",
    "Send": "发送",
    "Auto-scroll": "自动滚动",
    "Improve Prompt": "优化指示",
    "Semantic Searches": "语义搜索",
    "Frequently Used Resources": "常用资源",
    "User & Custom Data": "用户与自定数据",
    "Tabs Management": "标签页管理",
    "Appearance": "外观",
    "Enter your message...": "输入您的消息...",
    "Creating agent...": "创建代理中...",
    "Created!": "已创建！",
    "Error": "错误",
}